//---------------------------------------------------------------------
// File name  : tb.v
// Module name: tb
// Created by : 
// ---------------------------------------------------------------------
// Release history
// VERSION |  Date        | AUTHOR     |    DESCRIPTION
// ---------------------------------------------------------------------
//  1.0    |  1-June-2020 |            |     initial 
// ---------------------------------------------------------------------

`timescale 1ns / 1ps  

//According to IP parameters to choose
`define STANDARD_MODE
//`define SIMPLE_MODE

`ifdef STANDARD_MODE
	`define SCLKDIV_DEFAULT 8'd255 // if SPI Clock Divider = 0
    //`define SCLKDIV_DEFAULT 8'd0 // if SPI Clock Divider = 1
    //`define SCLKDIV_DEFAULT 8'd1 // if SPI Clock Divider = 2
    //......
    //`define SCLKDIV_DEFAULT n-1 // if SPI Clock Divider = n
`endif    

module tb();
//========================================================
//parameters
parameter MAIN_CLK_FREQ       = 50.000;//clock frequency, unit: MHz
parameter MAIN_CLK_PERIOD     = 1000.0/MAIN_CLK_FREQ; //unit: ns

`ifdef STANDARD_MODE
    parameter WN              = (`SCLKDIV_DEFAULT==8'hff) ? 1 : ((`SCLKDIV_DEFAULT+1)*2);
`else
    parameter WN              = 1;
`endif     
parameter WAIT_TIME           = 100000000*WN;

//=======================================================
reg  clk   ;      
reg  rst_n ;

reg  start_i     ;
wire right_o     ;

wire spi_clk_o ;
wire spi_cs_n_o;
wire spi_so_io ;
wire spi_si_io ;

// ahb reg 
wire [31:0] haddr_reg;
wire [31:0] hrdata_reg;
wire        hreadyin_reg;
wire        hreadyout_reg;
wire  [1:0] hresp_reg;
wire        hsel_reg;
wire  [1:0] htrans_reg;
wire        hwrite_reg;
wire [31:0] hwdata_reg;


//========================================================
GSR GSR(.GSRI(1'b1));

//==============================================  
initial begin
  $fsdbDumpfile("tb.fsdb");
  $fsdbDumpvars;
end

//============================================== 
//rst_n
initial
  begin
    rst_n=1'b0;
	start_i = 1'b1       ;
    
    #25;
    rst_n=1'b1;
	
	#5000; //test start
	start_i  = 1'b0         ;
	#100;
	start_i  = 1'b1         ;
	
	#WAIT_TIME;
    $finish;
end

//=====================================================
//clk
initial
begin
	clk = 1'b0;
end

always  #(MAIN_CLK_PERIOD/2.0) clk = ~clk;


//==================================================
//SPI Nor Flash Control
flash_test flash_test_inst      
(
    .clk          (clk),
    .rst_n        (rst_n),
	
	.right_o      (right_o ),
	
    .haddr_reg    (haddr_reg), //ahb reg mode
 	.hrdata_reg   (hrdata_reg),
 	.hreadyin_reg (hreadyin_reg),
 	.hreadyout_reg(hreadyout_reg),
 	.hresp_reg    (hresp_reg),
 	.hsel_reg     (hsel_reg),
 	.htrans_reg   (htrans_reg),
 	.hwdata_reg   (hwdata_reg),
 	.hwrite_reg   (hwrite_reg)
);

//===================================================
//SPI Nor Flash Controller
SPI_Nor_Flash_Interface_Top SPI_Nor_Flash_Interface_Top_inst
(
    .I_hclk         (clk  ), 
	.I_hresetn      (rst_n),
    .I_haddr_reg    (haddr_reg),//ahb reg mode 
 	.O_hrdata_reg   (hrdata_reg),
 	.I_hreadyin_reg (hreadyin_reg),
 	.O_hreadyout_reg(hreadyout_reg),
 	.O_hresp_reg    (hresp_reg),
 	.I_hsel_reg     (hsel_reg),
 	.I_htrans_reg   (htrans_reg),
 	.I_hwdata_reg   (hwdata_reg),
 	.I_hwrite_reg   (hwrite_reg),

	.I_spi_clock    (clk         ),
    .I_spi_rstn     (rst_n       ),
    .O_flash_ck     (spi_clk_o   ),
	.O_flash_cs_n   (spi_cs_n_o  ),
	.IO_flash_do    (spi_so_io   ),
	.IO_flash_di    (spi_si_io   )
);

//==============================================
//flash_model
spi_flash spi_flash_inst
(
	.SCLK           (spi_clk_o ),
	.CS             (spi_cs_n_o),
	.SI             (spi_si_io),
	.SO             (spi_so_io),
	.WP             (),
	.SIO3           (),
	.RESET          (rst_n) //low active
);
defparam spi_flash_inst.TOP_Add=27'h7ffffff;
defparam spi_flash_inst.init_file="../../tb/flash_model/spi_flash.dat";
          
endmodule
